<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Models\ActualGanttTask;
use App\Models\GanttTask;
use App\Models\GanttTaskResumeDaily;
use App\Models\Project;
use Illuminate\Support\Facades\Auth;
use DB;
use Illuminate\Support\Carbon;
use PDF;
use Illuminate\Support\Facades\Validator;

class DashboardController extends Controller
{
    
    public function __construct()
    {
        return $this->middleware('auth:web');
        
    }

    public function index(Request $request)
    {
        // $pr = Project::get();
        // Define validation rules
        $start_year = date('Y').'-01-01';
        $end_year = date('Y').'-12-31';

    $rules = [
        'start_date' => 'nullable|date|before_or_equal:end_date|after_or_equal:'.$start_year,
        'end_date' => 'nullable|date|after_or_equal:start_date|before_or_equal:'.$end_year,
    ];

    // Define custom validation messages
    $messages = [
        'start_date.before_or_equal' => 'The start date must be before or equal to the end date.',
        'end_date.after_or_equal' => 'The end date must be after or equal to the start date.',
        'start_date.after_or_equal' => 'The start date must be within the current year.',
        'end_date.before_or_equal' => 'The end date must be within the current year.',
    ];

    // Validate the request
    $validator = Validator::make($request->all(), $rules, $messages);

    // If validation fails, return the error messages
    if ($validator->fails()) {
        return redirect()->back()->withErrors($validator)->withInput();
    }
    
        $cekuserlogin = Auth::user()->employee;


        $prj = Project::where(function ($w) use ($cekuserlogin) {
            $w->where('status','Started')
                ->where('project_manager', $cekuserlogin->sso_email)
                ->orWhere('project_owner', $cekuserlogin->sso_email)
                ->orWhere(function ($q) use ($cekuserlogin) {
                    $q->whereIn('team_member', [$cekuserlogin->sso_email])
                    ->orWhere('team_member', 'LIKE', '%' . $cekuserlogin->sso_email . '%');
                })
                ->orWhere('created_by', $cekuserlogin->user_id);
        })
        ->orderBy('created_at', 'DESC')
        ->get();

        $pr = Project::where('status','Started')->get();
       
        $curdate = Carbon::now();
        $start_year = date('Y').'-01-01';
        $end_year = date('Y').'-12-31';
        $start_date = (!empty($request->start_date)? Carbon::parse($request->start_date)->format('Y-m-d') : '');
        $end_date = (!empty($request->end_date)? Carbon::parse($request->end_date)->format('Y-m-d') : '');
        
        $projects = Project::select('*')
            // whereHas('Ganttask', function($j) use ($start_year, $end_year){
            //     $j->where('sortable',0)->where(function($jw) use($start_year, $end_year){
            //         $jw->whereBetween('start',[$start_year, $end_year])->orwhereBetween('end',[$start_year, $end_year]);
            //     });
            // })
            
            ->with('Ganttask',function($w){
                $w->orderby('sortable');
                // $w->where('sortable',0);
            })
            ->with('GanttaskActual',function($w){
                $w->orderby('sortable');
                // $w->where('sortable',0);
            })
            ->when($request->has('project_id'), function($w) use($request){
                $w->whereIn('id', $request->project_id);
            })
            ->when($request->filled('start_date') and $request->filled('end_date'), function($w) use($start_date, $end_date){
                $w->whereBetween('start_date',[$start_date.' 00:00:00', $end_date.' 23:59:59'])->orwhereBetween('end_date',[$start_date.' 00:00:00', $end_date.' 23:59:59']);
                
            })
            ->when(count($request->all()) == 0, function($w) use($start_year, $end_year){
                $w->whereBetween('start_date',[$start_year.' 00:00:00', $end_year.' 23:59:59'])->orwhereBetween('end_date',[$start_year.' 00:00:00', $end_year.' 23:59:59']);
                
            })
            
            // ->where('status','Started')
            ->get();

        $data = array(
            'project' => 0,
            'not_started' => 0,
            'in_progress' => 0,
            'done' => 0
        );

        if($projects->isNotEmpty()){
            $not_started = $in_progress = $done = 0;
            
            $total_project = $projects->count();
            $data['project'] = $total_project;

            foreach($projects as $val){
                if($val->status != 'Started'){
                    $data['not_started'] = ++$not_started;
                }else{
                    $diff_awal = $curdate->diffInDays($val->GanttaskActual->first()->start,false);
                    $diff_akhir = $curdate->diffInDays($val->GanttaskActual->first()->end,false);
                    
                    if( $diff_awal < 0 ){
                        if($val->GanttaskActual->first()->progress == 100){
                            $data['done'] = ++$done;
                        }else{
                            $data['in_progress'] = ++$in_progress;
                        }
                    }else{
                        $data['not_started'] = ++$not_started;
                    }
                }
            }
        }

        
        
        $progress = $failed = $terlambat = $complete = $waiting = 0;
        $gaps_status = 0;
        
        $title_status=["On Progress","Failed","SUSPENDED","Completed", "Waiting"];

        $estimate_delivery = $total_project = $gap = $gap_status = $Planning = $Procurement = $Implementation = $Closing = 0;
        $progress = $progress_status = 0;
        $gap_cnt = $gap_status_cnt = $Planning_cnt = $Procurement_cnt = $Implementation_cnt = $Closing_cnt = 0;
        $progress_cnt = $gap_lbh_20_cnt = $gap_lbh_10_cnt = $gap_kurang_10_cnt = 0;
        $gap_note = 'ON TRACK';
        $gap_color = 'bg-success';

        $data_progress = array(
            'progress_status' => array(
                'Planning' => 0,
                'Procurement' => 0,
                'Implementation' => 0,
                'Closing' => 0,
            ),
            'progress' => 0,
            'estimate_delivery' => 0,
            'gap_status' => 0,
            'gap_color' => $gap_color,
            'gap_note' => $gap_note
        );

        $gaps = array(
            'estimate_delivery' => 0,
            'gap_status' => 0,
            'gap_color' => $gap_color,
            'gap_note' => $gap_note
        );

        $project_names = $project_names_s = $nama = $realisasi = $target = array();
        if($projects->isNotEmpty()){
            $project_ids = $projects->pluck('id');
            $total_project = $projects->count();

            //gap status
            $sub = GanttTaskResumeDaily::whereIn('project_id', $project_ids)
                ->whereNotNull('progress_daily_akumulasi_realisasi')
                ->selectRaw('project_id, max(days) as max_days')
                ->groupBy('project_id');

            $gap_status = GanttTaskResumeDaily::whereIn('gantt_task_resume_dailies.project_id', $project_ids)
                ->joinSub($sub, 'sub', function($j){
                    $j->on('sub.project_id','gantt_task_resume_dailies.project_id')
                        ->on('sub.max_days','gantt_task_resume_dailies.days');
                })
                ->join('projects','projects.id','gantt_task_resume_dailies.project_id')
                ->selectRaw("projects.project_name,gantt_task_resume_dailies.project_id, (progress_daily_akumulasi_realisasi - progress_daily_akumulasi_planning) as dif_progress,progress_daily_akumulasi_realisasi as realisasi, progress_daily_akumulasi_planning as planning")
                ->get();
            if($gap_status->isNotEmpty()){                
                $progress_status = $gap_status->pluck('realisasi')->sum();
                foreach($gap_status as $gproject){
                    $nama[] = $gproject->project_name;
                    $realisasi[] = round($gproject->realisasi);
                    $target[]= round($gproject->planning);
                    $gaps_status += round($gproject->realisasi) - round($gproject->planning);
                }            
                
                $gap = round($gaps_status/$gap_status->count());
                if($gap < -25){
                    $gap_color = 'bg-danger';
                    $gap_note = 'RISKS / ROADBLOCKS';
                }elseif($gap <= -15){
                    $gap_color = 'bg-warning';
                    $gap_note = 'POTENTIAL RISKS';
                }

            }
            
            $gap_q = GanttTask::join('gantt_task_actuals', 'gantt_task_actuals.project_id', 'gantt_tasks.project_id')
                ->where('gantt_task_actuals.sortable', 'gantt_tasks.sortable')
                ->whereIn('gantt_tasks.project_id', $project_ids)
                ->where('gantt_tasks.sortable', 0)
                ->selectRaw('gantt_tasks.end planning_end,gantt_tasks.duration planning_duration,
                    gantt_task_actuals.progress actual_progress,gantt_task_actuals.end actual_end,gantt_task_actuals.duration actual_duration')
                ->get();
            if($gap_q->isNotEmpty()){
                
                foreach($gap_q as $val_gap){
                    $est_gap = Carbon::parse($val_gap->planning_end)->diffInDays($val_gap->actual_end,false);
                    if($est_gap > 0){
                        $estimate_delivery += $est_gap;
                    }
                }
            }
            
           

            foreach($projects as $val){
                
                if($val->status != 'Started'){
                    $project_names[] = '<span class="badge bg-danger">'.$val->project_name.'</span> <span class="badge" style="color: #dededf;">(Not Started)</span>';
                }
                if($total_project == 1){
                    if($val->status == 'Started'){
                        $aktual = $val->GanttaskActual->where('sortable','>',0);
                        $progress = $aktual->where('status','STATUS_ACTIVE')->count();
                        $failed = $aktual->where('status','STATUS_FAILED')->count();
                        $terlambat = $aktual->where('status','STATUS_SUSPENDED')->count();
                        $complete = $aktual->where('status','STATUS_DONE')->count();
                        $waiting = $aktual->whereIn('status',['STATUS_UNDEFINED','STATUS_WAITING'])->count();

                        $aktual = $val->GanttaskActual->where('sortable','>',1);
                        $Planning = ( $aktual->where('code','Planning')->count() == 0?0:round($aktual->where('code','Planning')->sum('progress')/$aktual->where('code','Planning')->count()) );
                        $Procurement = ( $aktual->where('code','Procurement')->count() == 0?0:round($aktual->where('code','Procurement')->sum('progress')/$aktual->where('code','Procurement')->count()) );
                        $Implementation = ( $aktual->where('code','Implementation')->count() == 0?0:round($aktual->where('code','Implementation')->sum('progress')/$aktual->where('code','Implementation')->count()) );
                        $Closing = ( $aktual->where('code','Closing')->count() == 0?0:round($aktual->where('code','Closing')->sum('progress')/$aktual->where('code','Closing')->count()) );
                        
                        $project_names[] = '<span class="badge bg-danger">'.$val->project_name.'</span>';
                        if($request->has('project_id')){
                            $project_names_s[] = '<h4><span class="badge bg-danger">'.$val->project_name.'</span></h4>';
                        }
                    }
                }else{
                    if($val->status == 'Started'){
                        if($val->GanttaskActual->first()->status == 'STATUS_ACTIVE'){
                            ++$progress;
                            $status_p = '<span class="badge" style="color: #3BBF67;">on progress</span>';
                        }elseif($val->GanttaskActual->first()->status == 'STATUS_FAILED'){
                            ++$failed;
                            $status_p = '<span class="badge" style="color: #763A96;">failed</span>';
                        }elseif($val->GanttaskActual->first()->status == 'STATUS_SUSPENDED'){
                            ++$terlambat;
                            $status_p = '<span class="badge" style="color: #F9C154;">suspended</span>';
                        }elseif($val->GanttaskActual->first()->status == 'STATUS_DONE'){
                            ++$complete;
                            $status_p = '<span class="badge" style="color: #6EBEF4;">done</span>';
                        }elseif(in_array($val->GanttaskActual->first()->status, ['STATUS_UNDEFINED','STATUS_WAITING']) ){
                            ++$waiting;
                            $status_p = '<span class="badge" style="color: #dededf;">waiting</span>';
                        }
                        
                        $aktual = $val->GanttaskActual->where('sortable','>',1);
                        $Planning += ($aktual->where('code','Planning')->count() == 0?0:round($aktual->where('code','Planning')->sum('progress')/$aktual->where('code','Planning')->count()));
                        $Procurement += ($aktual->where('code','Procurement')->count() == 0?0:round($aktual->where('code','Procurement')->sum('progress')/$aktual->where('code','Procurement')->count()));
                        $Implementation += ($aktual->where('code','Implementation')->count() == 0?0:round($aktual->where('code','Implementation')->sum('progress')/$aktual->where('code','Implementation')->count()));
                        $Closing += ($aktual->where('code','Closing')->count() == 0?0:round($aktual->where('code','Closing')->sum('progress')/$aktual->where('code','Closing')->count()));

                        $project_names[] = '<span class="badge bg-danger">'.$val->project_name.'</span> ('.$status_p.')';
                        if($request->has('project_id')){
                            $project_names_s[] = '<h4><span class="badge bg-danger">'.$val->project_name.'</span></h4>';
                        }
                    }
                }
            }

            $data_progress = array(
                'progress_status' => array(
                    'Planning' => $Planning?round($Planning/$total_project):0,
                    'Procurement' => $Procurement?round($Procurement/$total_project):0,
                    'Implementation' => $Implementation?round($Implementation/$total_project):0,
                    'Closing' => $Closing?round($Closing/$total_project):0,
                ),
                'progress' => round($progress_status/$total_project),
                'estimate_delivery' => round($estimate_delivery/$total_project),
                'gap_status' => (substr($gap,0,1)=="-"? $gap: "+".$gap),
                'gap_color' => $gap_color,
                'gap_note' => $gap_note
            );
        }
        
        $dat=[$progress,$failed,$terlambat,$complete, $waiting];
        $status_color = json_encode(array('#3BBF67','#763A96','#F9C154','#6EBEF4','#dededf'));
        
        $costs = Project::leftJoin('gantt_tasks','projects.id','gantt_tasks.project_id')
            ->leftJoin('gantt_task_actuals','projects.id','gantt_task_actuals.project_id')
            ->where('gantt_task_actuals.level',0)
            ->where('gantt_tasks.level',0)
            ->when(!empty($request->project_id), function($w)use($request){
                $w->whereIn('projects.id',$request->project_id);
            })                         
            ->when($request->filled('start_date') and $request->filled('end_date'), function($w) use($start_date, $end_date){
                $w->whereBetween('start_date',[$start_date.' 00:00:00', $end_date.' 23:59:59'])->orwhereBetween('end_date',[$start_date.' 00:00:00', $end_date.' 23:59:59']);
                
            })
            ->when(count($request->all()) == 0, function($w) use($start_year, $end_year){
                $w->whereBetween('start_date',[$start_year.' 00:00:00', $end_year.' 23:59:59'])->orwhereBetween('end_date',[$start_year.' 00:00:00', $end_year.' 23:59:59']);
                
            })                       
            ->selectRaw("projects.project_name, SUM(gantt_task_actuals.task_budget) as actual, SUM(gantt_tasks.task_budget) as plan,
                projects.project_budget as budget")
            ->groupBy('projects.id')->get();

        //status proyek


        $costnama=[];
        $costactual=[];
        $costplan=[];
        $costbudget=[];
        foreach($costs as $cost)
        {
            $costnama[]=$cost->project_name;
            $costactual[]= (int)$cost->actual;
            $costplan[]= (int)$cost->plan;
            $costbudget[]=$cost->budget;
        }
        
        $qs = explode('&',$request->getRequestUri(),2);
        return view('admin.dashboard.index',compact('data','project_names','data_progress','pr','dat','title_status',
                'nama','target','realisasi','costnama','costactual','costplan','costbudget','project_names_s','qs','status_color','prj'));
    }

    public function dashboard_employee(Request $request)
    {
        $start_year = date('Y').'-01-01';
        $end_year = date('Y').'-12-31';

        $rules = [
            'start_date' => 'nullable|date|before_or_equal:end_date|after_or_equal:'.$start_year,
            'end_date' => 'nullable|date|after_or_equal:start_date|before_or_equal:'.$end_year,
        ];

        // Define custom validation messages
        $messages = [
            'start_date.before_or_equal' => 'The start date must be before or equal to the end date.',
            'end_date.after_or_equal' => 'The end date must be after or equal to the start date.',
            'start_date.after_or_equal' => 'The start date must be within the current year.',
            'end_date.before_or_equal' => 'The end date must be within the current year.',
        ];

        // Validate the request
        $validator = Validator::make($request->all(), $rules, $messages);

        // If validation fails, return the error messages
        if ($validator->fails()) {
            return redirect()->back()->withErrors($validator)->withInput();
        }

        $cekuserlogin = Auth::user()->employee;

        // $pr = Project::where('status','Started')->get();
        $pr = Project::where(function ($w) use ($cekuserlogin) {
            $w->where('status','Started')
                ->where('project_manager', $cekuserlogin->sso_email)
                ->orWhere('project_owner', $cekuserlogin->sso_email)
                ->orWhere(function ($q) use ($cekuserlogin) {
                    $q->whereIn('team_member', [$cekuserlogin->sso_email])
                      ->orWhere('team_member', 'LIKE', '%' . $cekuserlogin->sso_email . '%');
                })
                ->orWhere('created_by', $cekuserlogin->user_id);
        })
        ->orderBy('created_at', 'DESC')
        ->get();
       
        $curdate = Carbon::now();
        $start_year = date('Y').'-01-01';
        $end_year = date('Y').'-12-31';
        $start_date = (!empty($request->start_date)? Carbon::parse($request->start_date)->format('Y-m-d') : '');
        $end_date = (!empty($request->end_date)? Carbon::parse($request->end_date)->format('Y-m-d') : '');

        $projects = Project::where(function ($w) use ($cekuserlogin) {
            $w->where('project_manager', $cekuserlogin->sso_email)
                ->orWhere('project_owner', $cekuserlogin->sso_email)
                ->orWhere(function ($q) use ($cekuserlogin) {
                    $q->whereIn('team_member', [$cekuserlogin->sso_email])
                      ->orWhere('team_member', 'LIKE', '%' . $cekuserlogin->sso_email . '%');
                })
                ->orWhere('created_by', $cekuserlogin->user_id);
        })
        ->orderBy('created_at', 'DESC')
        ->get();
        
        // dd($projects);
        
        $project = Project::select('*')
           ->with('Ganttask',function($w){
                $w->orderby('sortable');
                
            })
            ->with('GanttaskActual',function($w){
                $w->orderby('sortable');
                
            })
            ->when($request->has('project_id'), function($w) use($request){
                $w->whereIn('id', $request->project_id);
            })
            ->when($request->filled('start_date') and $request->filled('end_date'), function($w) use($start_date, $end_date){
                $w->whereBetween('start_date',[$start_date.' 00:00:00', $end_date.' 23:59:59'])->orwhereBetween('end_date',[$start_date.' 00:00:00', $end_date.' 23:59:59']);
                
            })
            ->when(count($request->all()) == 0, function($w) use($start_year, $end_year){
                $w->whereBetween('start_date',[$start_year.' 00:00:00', $end_year.' 23:59:59'])->orwhereBetween('end_date',[$start_year.' 00:00:00', $end_year.' 23:59:59']);
                
            })
            
            
            ->get();


        $data = array(
            'project' => 0,
            'not_started' => 0,
            'in_progress' => 0,
            'done' => 0
        );

        if($projects->isNotEmpty()){
            $not_started = $in_progress = $done = 0;
            
            $total_project = $projects->count();
            $data['project'] = $total_project;

            foreach($projects as $val){
                if($val->status != 'Started'){
                    $data['not_started'] = ++$not_started;
                }else{
                    $diff_awal = $curdate->diffInDays($val->GanttaskActual->first()->start,false);
                    $diff_akhir = $curdate->diffInDays($val->GanttaskActual->first()->end,false);
                    
                    if( $diff_awal < 0 ){
                        if($val->GanttaskActual->first()->progress == 100){
                            $data['done'] = ++$done;
                        }else{
                            $data['in_progress'] = ++$in_progress;
                        }
                    }else{
                        $data['not_started'] = ++$not_started;
                    }
                }
            }
        }

        $progress = $failed = $terlambat = $complete = $waiting = 0;
        $gaps_status = 0;
        
        $title_status=["On Progress","Failed","SUSPENDED","Completed", "Waiting"];

        $estimate_delivery = $total_project = $gap = $gap_status = $Planning = $Procurement = $Implementation = $Closing = 0;
        $progress = $progress_status = 0;
        $gap_cnt = $gap_status_cnt = $Planning_cnt = $Procurement_cnt = $Implementation_cnt = $Closing_cnt = 0;
        $progress_cnt = $gap_lbh_20_cnt = $gap_lbh_10_cnt = $gap_kurang_10_cnt = 0;
        $gap_note = 'ON TRACK';
        $gap_color = 'bg-success';

        $data_progress = array(
            'progress_status' => array(
                'Planning' => 0,
                'Procurement' => 0,
                'Implementation' => 0,
                'Closing' => 0,
            ),
            'progress' => 0,
            'estimate_delivery' => 0,
            'gap_status' => 0,
            'gap_color' => $gap_color,
            'gap_note' => $gap_note
        );

        $gaps = array(
            'estimate_delivery' => 0,
            'gap_status' => 0,
            'gap_color' => $gap_color,
            'gap_note' => $gap_note
        );

        $project_names = $project_names_s = $nama = $realisasi = $target = array();
        if($projects->isNotEmpty()){
            $project_ids = $projects->pluck('id');
            $total_project = $projects->count();

            //gap status
            $sub = GanttTaskResumeDaily::whereIn('project_id', $project_ids)
                ->whereNotNull('progress_daily_akumulasi_realisasi')
                ->selectRaw('project_id, max(days) as max_days')
                ->groupBy('project_id');

            $gap_status = GanttTaskResumeDaily::whereIn('gantt_task_resume_dailies.project_id', $project_ids)
                ->joinSub($sub, 'sub', function($j){
                    $j->on('sub.project_id','gantt_task_resume_dailies.project_id')
                        ->on('sub.max_days','gantt_task_resume_dailies.days');
                })
                ->join('projects','projects.id','gantt_task_resume_dailies.project_id')
                ->selectRaw("projects.project_name,gantt_task_resume_dailies.project_id, (progress_daily_akumulasi_realisasi - progress_daily_akumulasi_planning) as dif_progress,progress_daily_akumulasi_realisasi as realisasi, progress_daily_akumulasi_planning as planning")
                ->get();
            if($gap_status->isNotEmpty()){                
                $progress_status = $gap_status->pluck('realisasi')->sum();
                foreach($gap_status as $gproject){
                    $nama[] = $gproject->project_name;
                    $realisasi[] = round($gproject->realisasi);
                    $target[]= round($gproject->planning);
                    $gaps_status += round($gproject->realisasi) - round($gproject->planning);
                }            
                
                $gap = round($gaps_status/$gap_status->count());
                if($gap < -25){
                    $gap_color = 'bg-danger';
                    $gap_note = 'RISKS / ROADBLOCKS';
                }elseif($gap <= -15){
                    $gap_color = 'bg-warning';
                    $gap_note = 'POTENTIAL RISKS';
                }

            }
            
            $gap_q = GanttTask::join('gantt_task_actuals', 'gantt_task_actuals.project_id', 'gantt_tasks.project_id')
                ->where('gantt_task_actuals.sortable', 'gantt_tasks.sortable')
                ->whereIn('gantt_tasks.project_id', $project_ids)
                ->where('gantt_tasks.sortable', 0)
                ->selectRaw('gantt_tasks.end planning_end,gantt_tasks.duration planning_duration,
                    gantt_task_actuals.progress actual_progress,gantt_task_actuals.end actual_end,gantt_task_actuals.duration actual_duration')
                ->get();
            if($gap_q->isNotEmpty()){
                
                foreach($gap_q as $val_gap){
                    $est_gap = Carbon::parse($val_gap->planning_end)->diffInDays($val_gap->actual_end,false);
                    if($est_gap > 0){
                        $estimate_delivery += $est_gap;
                    }
                }
            }
            
           

            foreach($projects as $val){
                
                if($val->status != 'Started'){
                    $project_names[] = '<span class="badge bg-danger">'.$val->project_name.'</span> <span class="badge" style="color: #dededf;">(Not Started)</span>';
                }
                if($total_project == 1){
                    if($val->status == 'Started'){
                        $aktual = $val->GanttaskActual->where('sortable','>',0);
                        $progress = $aktual->where('status','STATUS_ACTIVE')->count();
                        $failed = $aktual->where('status','STATUS_FAILED')->count();
                        $terlambat = $aktual->where('status','STATUS_SUSPENDED')->count();
                        $complete = $aktual->where('status','STATUS_DONE')->count();
                        $waiting = $aktual->whereIn('status',['STATUS_UNDEFINED','STATUS_WAITING'])->count();

                        $aktual = $val->GanttaskActual->where('sortable','>',1);
                        $Planning = ( $aktual->where('code','Planning')->count() == 0?0:round($aktual->where('code','Planning')->sum('progress')/$aktual->where('code','Planning')->count()) );
                        $Procurement = ( $aktual->where('code','Procurement')->count() == 0?0:round($aktual->where('code','Procurement')->sum('progress')/$aktual->where('code','Procurement')->count()) );
                        $Implementation = ( $aktual->where('code','Implementation')->count() == 0?0:round($aktual->where('code','Implementation')->sum('progress')/$aktual->where('code','Implementation')->count()) );
                        $Closing = ( $aktual->where('code','Closing')->count() == 0?0:round($aktual->where('code','Closing')->sum('progress')/$aktual->where('code','Closing')->count()) );
                        
                        $project_names[] = '<span class="badge bg-danger">'.$val->project_name.'</span>';
                        if($request->has('project_id')){
                            $project_names_s[] = '<h4><span class="badge bg-danger">'.$val->project_name.'</span></h4>';
                        }
                    }
                }else{
                    if($val->status == 'Started'){
                        if($val->GanttaskActual->first()->status == 'STATUS_ACTIVE'){
                            ++$progress;
                            $status_p = '<span class="badge" style="color: #3BBF67;">on progress</span>';
                        }elseif($val->GanttaskActual->first()->status == 'STATUS_FAILED'){
                            ++$failed;
                            $status_p = '<span class="badge" style="color: #763A96;">failed</span>';
                        }elseif($val->GanttaskActual->first()->status == 'STATUS_SUSPENDED'){
                            ++$terlambat;
                            $status_p = '<span class="badge" style="color: #F9C154;">suspended</span>';
                        }elseif($val->GanttaskActual->first()->status == 'STATUS_DONE'){
                            ++$complete;
                            $status_p = '<span class="badge" style="color: #6EBEF4;">done</span>';
                        }elseif(in_array($val->GanttaskActual->first()->status, ['STATUS_UNDEFINED','STATUS_WAITING']) ){
                            ++$waiting;
                            $status_p = '<span class="badge" style="color: #dededf;">waiting</span>';
                        }
                        
                        $aktual = $val->GanttaskActual->where('sortable','>',1);
                        $Planning += ($aktual->where('code','Planning')->count() == 0?0:round($aktual->where('code','Planning')->sum('progress')/$aktual->where('code','Planning')->count()));
                        $Procurement += ($aktual->where('code','Procurement')->count() == 0?0:round($aktual->where('code','Procurement')->sum('progress')/$aktual->where('code','Procurement')->count()));
                        $Implementation += ($aktual->where('code','Implementation')->count() == 0?0:round($aktual->where('code','Implementation')->sum('progress')/$aktual->where('code','Implementation')->count()));
                        $Closing += ($aktual->where('code','Closing')->count() == 0?0:round($aktual->where('code','Closing')->sum('progress')/$aktual->where('code','Closing')->count()));

                        $project_names[] = '<span class="badge bg-danger">'.$val->project_name.'</span> ('.$status_p.')';
                        if($request->has('project_id')){
                            $project_names_s[] = '<h4><span class="badge bg-danger">'.$val->project_name.'</span></h4>';
                        }
                    }
                }
            }

            $data_progress = array(
                'progress_status' => array(
                    'Planning' => $Planning?round($Planning/$total_project):0,
                    'Procurement' => $Procurement?round($Procurement/$total_project):0,
                    'Implementation' => $Implementation?round($Implementation/$total_project):0,
                    'Closing' => $Closing?round($Closing/$total_project):0,
                ),
                'progress' => round($progress_status/$total_project),
                'estimate_delivery' => round($estimate_delivery/$total_project),
                'gap_status' => (substr($gap,0,1)=="-"? $gap: "+".$gap),
                'gap_color' => $gap_color,
                'gap_note' => $gap_note
            );
        }
        
        $dat=[$progress,$failed,$terlambat,$complete, $waiting];
        $status_color = json_encode(array('#3BBF67','#763A96','#F9C154','#6EBEF4','#dededf'));
        
        $costs = Project::leftJoin('gantt_tasks','projects.id','gantt_tasks.project_id')
            ->leftJoin('gantt_task_actuals','projects.id','gantt_task_actuals.project_id')
            ->where('gantt_task_actuals.level',0)
            ->where('gantt_tasks.level',0)
            ->when(!empty($request->project_id), function($w)use($request){
                $w->whereIn('projects.id',$request->project_id);
            })                         
            ->when($request->filled('start_date') and $request->filled('end_date'), function($w) use($start_date, $end_date){
                $w->whereBetween('start_date',[$start_date.' 00:00:00', $end_date.' 23:59:59'])->orwhereBetween('end_date',[$start_date.' 00:00:00', $end_date.' 23:59:59']);
                
            })
            ->when(count($request->all()) == 0, function($w) use($start_year, $end_year){
                $w->whereBetween('start_date',[$start_year.' 00:00:00', $end_year.' 23:59:59'])->orwhereBetween('end_date',[$start_year.' 00:00:00', $end_year.' 23:59:59']);
                
            })                       
            ->selectRaw("projects.project_name, SUM(gantt_task_actuals.task_budget) as actual, SUM(gantt_tasks.task_budget) as plan,
                projects.project_budget as budget")
            ->groupBy('projects.id')->get();

        //status proyek


        $costnama=[];
        $costactual=[];
        $costplan=[];
        $costbudget=[];
        foreach($costs as $cost)
        {
            $costnama[]=$cost->project_name;
            $costactual[]= (int)$cost->actual;
            $costplan[]= (int)$cost->plan;
            $costbudget[]=$cost->budget;
        }
        
        $qs = explode('&',$request->getRequestUri(),2);

        return view('admin.dashboard.dashboard_employee', compact('data','project_names','data_progress','pr','dat','title_status',
        'nama','target','realisasi','costnama','costactual','costplan','costbudget','project_names_s','qs','status_color'));
    }

    public function markAllAsRead()
    {
        Auth::user()->unreadNotifications->markAsRead();
        return redirect()->back();
    }

    public function seeallnotif()
    {
        $user = Auth::user();
        return view('admin.notification.index')->with('notifications', $user->notifications);
    }

    public function seeallProgress(Request $request)
    {
        $pr = Project::where('status','Started')->get();
       
        $curdate = Carbon::now();
        $start_year = date('Y').'-01-01';
        $end_year = date('Y').'-12-31';
        $start_date = (!empty($request->start_date)? Carbon::parse($request->start_date)->format('Y-m-d') : '');
        $end_date = (!empty($request->start_date)? Carbon::parse($request->end_date)->format('Y-m-d') : '');

        
        $projects = Project::select('*')
            // whereHas('Ganttask', function($j) use ($start_year, $end_year){
            //     $j->where('sortable',0)->where(function($jw) use($start_year, $end_year){
            //         $jw->whereBetween('start',[$start_year, $end_year])->orwhereBetween('end',[$start_year, $end_year]);
            //     });
            // })
            
            ->with('Ganttask',function($w){
                $w->orderby('sortable');
                // $w->where('sortable',0);
            })
            ->with('GanttaskActual',function($w){
                $w->orderby('sortable');
                // $w->where('sortable',0);
            })
            ->when($request->has('project_id'), function($w) use($request){
                $w->whereIn('id', $request->project_id);
            })
            ->when($request->filled('start_date') and $request->filled('end_date'), function($w) use($start_date, $end_date){
                $w->whereBetween('start_date',[$start_date.' 00:00:00', $end_date.' 23:59:59'])->orwhereBetween('end_date',[$start_date.' 00:00:00', $end_date.' 23:59:59']);
                
            })
            ->when(count($request->all()) == 0, function($w) use($start_year, $end_year){
                $w->whereBetween('start_date',[$start_year.' 00:00:00', $end_year.' 23:59:59'])->orwhereBetween('end_date',[$start_year.' 00:00:00', $end_year.' 23:59:59']);
                
            })
            
            // ->where('status','Started')
            ->get();

        $data = array(
            'project' => 0,
            'not_started' => 0,
            'in_progress' => 0,
            'done' => 0
        );

        if($projects->isNotEmpty()){
            $not_started = $in_progress = $done = 0;
            
            $total_project = $projects->count();
            $data['project'] = $total_project;

            foreach($projects as $val){
                if($val->status != 'Started'){
                    $data['not_started'] = ++$not_started;
                }else{
                    $diff_awal = $curdate->diffInDays($val->GanttaskActual->first()->start,false);
                    $diff_akhir = $curdate->diffInDays($val->GanttaskActual->first()->end,false);
                    
                    if( $diff_awal < 0 ){
                        if($val->GanttaskActual->first()->progress == 100){
                            $data['done'] = ++$done;
                        }else{
                            $data['in_progress'] = ++$in_progress;
                        }
                    }else{
                        $data['not_started'] = ++$not_started;
                    }
                }
            }
        }

        /*$projects = Project::select('*')
            ->with('Ganttask',function($w){
                $w->orderby('sortable');
                // $w->where('sortable',0);
            })
            ->with('GanttaskActual',function($w){
                $w->orderby('sortable');
                // $w->where('sortable',0);
            })
            ->when($request->has('project_id'), function($w) use($request){
                $w->whereIn('id', $request->project_id);
            })
            ->when($request->filled('start_date') and $request->filled('end_date'), function($w) use($start_date, $end_date){
                $w->whereBetween('start_date',[$start_date, $end_date])->orwhereBetween('end_date',[$start_date, $end_date]);
                
            })
            ->when(count($request->all()) == 0, function($w) use($start_year, $end_year){
                $w->whereBetween('start_date',[$start_year, $end_year])->orwhereBetween('end_date',[$start_year, $end_year]);
                
            })
            ->where('status','Started')->get();*/
        
        $progress = $failed = $terlambat = $complete = $waiting = 0;
        $gaps_status = 0;
        
        $title_status=["On Progress","Failed","SUSPENDED","Completed", "Waiting"];

        $estimate_delivery = $total_project = $gap = $gap_status = $Planning = $Procurement = $Implementation = $Closing = 0;
        $progress = $progress_status = 0;
        $gap_cnt = $gap_status_cnt = $Planning_cnt = $Procurement_cnt = $Implementation_cnt = $Closing_cnt = 0;
        $progress_cnt = $gap_lbh_20_cnt = $gap_lbh_10_cnt = $gap_kurang_10_cnt = 0;
        $gap_note = 'ON TRACK';
        $gap_color = 'bg-success';

        $data_progress = array(
            'progress_status' => array(
                'Planning' => 0,
                'Procurement' => 0,
                'Implementation' => 0,
                'Closing' => 0,
            ),
            'progress' => 0,
            'estimate_delivery' => 0,
            'gap_status' => 0,
            'gap_color' => $gap_color,
            'gap_note' => $gap_note
        );

        $gaps = array(
            'estimate_delivery' => 0,
            'gap_status' => 0,
            'gap_color' => $gap_color,
            'gap_note' => $gap_note
        );

        $project_names = $nama = $realisasi = $target = array();
        if($projects->isNotEmpty()){
            $project_ids = $projects->pluck('id');
            $total_project = $projects->count();

            //gap status
            $sub = GanttTaskResumeDaily::whereIn('project_id', $project_ids)
                ->whereNotNull('progress_daily_akumulasi_realisasi')
                ->selectRaw('project_id, max(days) as max_days')
                ->groupBy('project_id');

            $gap_status = GanttTaskResumeDaily::whereIn('gantt_task_resume_dailies.project_id', $project_ids)
                ->joinSub($sub, 'sub', function($j){
                    $j->on('sub.project_id','gantt_task_resume_dailies.project_id')
                        ->on('sub.max_days','gantt_task_resume_dailies.days');
                })
                ->join('projects','projects.id','gantt_task_resume_dailies.project_id')
                ->selectRaw("projects.project_name,gantt_task_resume_dailies.project_id, (progress_daily_akumulasi_realisasi - progress_daily_akumulasi_planning) as dif_progress,progress_daily_akumulasi_realisasi as realisasi, progress_daily_akumulasi_planning as planning")
                ->get();
            if($gap_status->isNotEmpty()){                
                $progress_status = $gap_status->pluck('realisasi')->sum();
                foreach($gap_status as $gproject){
                    $nama[] = $gproject->project_name;
                    $realisasi[] = round($gproject->realisasi);
                    $target[]= round($gproject->planning);

                    $nama[] = $gproject->project_name;
                    $realisasi[] = round($gproject->realisasi);
                    $target[]= round($gproject->planning);

                    $nama[] = $gproject->project_name;
                    $realisasi[] = round($gproject->realisasi);
                    $target[]= round($gproject->planning);

                    $gaps_status += round($gproject->realisasi) - round($gproject->planning);
                }            
                
                $gap = round($gaps_status/$gap_status->count());
                if($gap < -20){
                    $gap_color = 'bg-danger';
                    $gap_note = 'RISKS / ROADBLOCKS';
                }elseif($gap <= -10){
                    $gap_color = 'bg-warning';
                    $gap_note = 'POTENTIAL RISKS';
                }

            }
            
            $gap_q = GanttTask::join('gantt_task_actuals', 'gantt_task_actuals.project_id', 'gantt_tasks.project_id')
                ->where('gantt_task_actuals.sortable', 'gantt_tasks.sortable')
                ->whereIn('gantt_tasks.project_id', $project_ids)
                ->where('gantt_tasks.sortable', 0)
                ->selectRaw('gantt_tasks.end planning_end,gantt_tasks.duration planning_duration,
                    gantt_task_actuals.progress actual_progress,gantt_task_actuals.end actual_end,gantt_task_actuals.duration actual_duration')
                ->get();
            if($gap_q->isNotEmpty()){
                
                foreach($gap_q as $val_gap){
                    $est_gap = Carbon::parse($val_gap->planning_end)->diffInDays($val_gap->actual_end,false);
                    if($est_gap > 0){
                        $estimate_delivery += $est_gap;
                    }
                }
            }
            
           

            foreach($projects as $val){
                
                if($val->status != 'Started'){
                    $project_names[] = '<span class="badge bg-danger">'.$val->project_name.'</span> <span class="badge" style="color: #dededf;">(Not Started)</span>';
                }
                if($total_project == 1){
                    if($val->status == 'Started'){
                        $aktual = $val->GanttaskActual->where('sortable','>',0);
                        $progress = $aktual->where('status','STATUS_ACTIVE')->count();
                        $failed = $aktual->where('status','STATUS_FAILED')->count();
                        $terlambat = $aktual->where('status','STATUS_SUSPENDED')->count();
                        $complete = $aktual->where('status','STATUS_DONE')->count();
                        $waiting = $aktual->whereIn('status',['STATUS_UNDEFINED','STATUS_WAITING'])->count();

                        $aktual = $val->GanttaskActual->where('sortable','>',1);
                        $Planning = ( $aktual->where('code','Planning')->count() == 0?0:round($aktual->where('code','Planning')->sum('progress')/$aktual->where('code','Planning')->count()) );
                        $Procurement = ( $aktual->where('code','Procurement')->count() == 0?0:round($aktual->where('code','Procurement')->sum('progress')/$aktual->where('code','Procurement')->count()) );
                        $Implementation = ( $aktual->where('code','Implementation')->count() == 0?0:round($aktual->where('code','Implementation')->sum('progress')/$aktual->where('code','Implementation')->count()) );
                        $Closing = ( $aktual->where('code','Closing')->count() == 0?0:round($aktual->where('code','Closing')->sum('progress')/$aktual->where('code','Closing')->count()) );
                        
                        $project_names[] = '<span class="badge bg-danger">'.$val->project_name.'</span>';
                        if($request->has('project_id')){
                            $project_names_s[] = '<h4><span class="badge bg-danger">'.$val->project_name.'</span></h4>';
                        }
                    }
                }else{
                    if($val->status == 'Started'){
                        if($val->GanttaskActual->first()->status == 'STATUS_ACTIVE'){
                            ++$progress;
                            $status_p = '<span class="badge" style="color: #3BBF67;">on progress</span>';
                        }elseif($val->GanttaskActual->first()->status == 'STATUS_FAILED'){
                            ++$failed;
                            $status_p = '<span class="badge" style="color: #763A96;">failed</span>';
                        }elseif($val->GanttaskActual->first()->status == 'STATUS_SUSPENDED'){
                            ++$terlambat;
                            $status_p = '<span class="badge" style="color: #F9C154;">suspended</span>';
                        }elseif($val->GanttaskActual->first()->status == 'STATUS_DONE'){
                            ++$complete;
                            $status_p = '<span class="badge" style="color: #6EBEF4;">done</span>';
                        }elseif(in_array($val->GanttaskActual->first()->status, ['STATUS_UNDEFINED','STATUS_WAITING']) ){
                            ++$waiting;
                            $status_p = '<span class="badge" style="color: #dededf;">waiting</span>';
                        }
                        
                        $aktual = $val->GanttaskActual->where('sortable','>',1);
                        $Planning += ($aktual->where('code','Planning')->count() == 0?0:round($aktual->where('code','Planning')->sum('progress')/$aktual->where('code','Planning')->count()));
                        $Procurement += ($aktual->where('code','Procurement')->count() == 0?0:round($aktual->where('code','Procurement')->sum('progress')/$aktual->where('code','Procurement')->count()));
                        $Implementation += ($aktual->where('code','Implementation')->count() == 0?0:round($aktual->where('code','Implementation')->sum('progress')/$aktual->where('code','Implementation')->count()));
                        $Closing += ($aktual->where('code','Closing')->count() == 0?0:round($aktual->where('code','Closing')->sum('progress')/$aktual->where('code','Closing')->count()));

                        $project_names[] = '<span class="badge bg-danger">'.$val->project_name.'</span> ('.$status_p.')';
                        if($request->has('project_id')){
                            $project_names_s[] = '<h4><span class="badge bg-danger">'.$val->project_name.'</span></h4>';
                        }
                    }
                }
            }

            $data_progress = array(
                'progress_status' => array(
                    'Planning' => $Planning?round($Planning/$total_project):0,
                    'Procurement' => $Procurement?round($Procurement/$total_project):0,
                    'Implementation' => $Implementation?round($Implementation/$total_project):0,
                    'Closing' => $Closing?round($Closing/$total_project):0,
                ),
                'progress' => round($progress_status/$total_project),
                'estimate_delivery' => round($estimate_delivery/$total_project),
                'gap_status' => (substr($gap,0,1)=="-"? $gap: "+".$gap),
                'gap_color' => $gap_color,
                'gap_note' => $gap_note
            );
        }
        
        $dat=[$progress,$failed,$terlambat,$complete, $waiting];
        
        $costs = Project::leftJoin('gantt_tasks','projects.id','gantt_tasks.project_id')
            ->leftJoin('gantt_task_actuals','projects.id','gantt_task_actuals.project_id')
            ->where('gantt_task_actuals.level',0)
            ->where('gantt_tasks.level',0)
            ->when(!empty($request->project_id), function($w)use($request){
                $w->whereIn('projects.id',$request->project_id);
            })                         
            ->when(!empty($start_date), function($w)use($start_date, $end_date){
                $w->whereBetween('projects.created_at',[$start_date.' 00:00:00',$end_date.' 23:59:59']);
            })                         
            ->selectRaw("projects.project_name, SUM(gantt_task_actuals.task_budget) as actual, SUM(gantt_tasks.task_budget) as plan,
                projects.project_budget as budget")
            ->groupBy('projects.id')->get();

        //status proyek


        $costnama=[];
        $costactual=[];
        $costplan=[];
        $costbudget=[];
        foreach($costs as $cost)
        {
            $costnama[]=$cost->project_name;
            $costactual[]= (int)$cost->actual;
            $costplan[]= (int)$cost->plan;
            $costbudget[]=$cost->budget;
        }
        
        $sizechart = array(
            'minwidth' => "800px",
            'maxwidth' => "1000px",
            'width' => "900px",
            'chartminWidth' => "900"
        );
        if(count($costnama)>5){
            $sizechart = array(
                'minwidth' => "820px",
                'maxwidth' => "1900px",
                'width' => "1010px",
                'chartminWidth' => "2000"
            );
            
        }
        
        return view('admin.dashboard.seeallProgress',compact('data','nama','target','realisasi','title_status', 'sizechart'));
    }

    public function seeallCost(Request $request)
    {
        $pr = Project::where('status','Started')->get();
       
        $curdate = Carbon::now();
        $start_year = date('Y').'-01-01';
        $end_year = date('Y').'-12-31';
        $start_date = (!empty($request->start_date)? Carbon::parse($request->start_date)->format('Y-m-d') : '');
        $end_date = (!empty($request->start_date)? Carbon::parse($request->end_date)->format('Y-m-d') : '');

        
        $projects = Project::select('*')
            // whereHas('Ganttask', function($j) use ($start_year, $end_year){
            //     $j->where('sortable',0)->where(function($jw) use($start_year, $end_year){
            //         $jw->whereBetween('start',[$start_year, $end_year])->orwhereBetween('end',[$start_year, $end_year]);
            //     });
            // })
            
            ->with('Ganttask',function($w){
                $w->orderby('sortable');
                // $w->where('sortable',0);
            })
            ->with('GanttaskActual',function($w){
                $w->orderby('sortable');
                // $w->where('sortable',0);
            })
            ->when($request->has('project_id'), function($w) use($request){
                $w->whereIn('id', $request->project_id);
            })
            ->when($request->filled('start_date') and $request->filled('end_date'), function($w) use($start_date, $end_date){
                $w->whereBetween('start_date',[$start_date.' 00:00:00', $end_date.' 23:59:59'])->orwhereBetween('end_date',[$start_date.' 00:00:00', $end_date.' 23:59:59']);
                
            })
            ->when(count($request->all()) == 0, function($w) use($start_year, $end_year){
                $w->whereBetween('start_date',[$start_year.' 00:00:00', $end_year.' 23:59:59'])->orwhereBetween('end_date',[$start_year.' 00:00:00', $end_year.' 23:59:59']);
                
            })
            
            // ->where('status','Started')
            ->get();

        $data = array(
            'project' => 0,
            'not_started' => 0,
            'in_progress' => 0,
            'done' => 0
        );

        if($projects->isNotEmpty()){
            $not_started = $in_progress = $done = 0;
            
            $total_project = $projects->count();
            $data['project'] = $total_project;

            foreach($projects as $val){
                if($val->status != 'Started'){
                    $data['not_started'] = ++$not_started;
                }else{
                    $diff_awal = $curdate->diffInDays($val->GanttaskActual->first()->start,false);
                    $diff_akhir = $curdate->diffInDays($val->GanttaskActual->first()->end,false);
                    
                    if( $diff_awal < 0 ){
                        if($val->GanttaskActual->first()->progress == 100){
                            $data['done'] = ++$done;
                        }else{
                            $data['in_progress'] = ++$in_progress;
                        }
                    }else{
                        $data['not_started'] = ++$not_started;
                    }
                }
            }
        }

        /*$projects = Project::select('*')
            ->with('Ganttask',function($w){
                $w->orderby('sortable');
                // $w->where('sortable',0);
            })
            ->with('GanttaskActual',function($w){
                $w->orderby('sortable');
                // $w->where('sortable',0);
            })
            ->when($request->has('project_id'), function($w) use($request){
                $w->whereIn('id', $request->project_id);
            })
            ->when($request->filled('start_date') and $request->filled('end_date'), function($w) use($start_date, $end_date){
                $w->whereBetween('start_date',[$start_date, $end_date])->orwhereBetween('end_date',[$start_date, $end_date]);
                
            })
            ->when(count($request->all()) == 0, function($w) use($start_year, $end_year){
                $w->whereBetween('start_date',[$start_year, $end_year])->orwhereBetween('end_date',[$start_year, $end_year]);
                
            })
            ->where('status','Started')->get();*/
        
        $progress = $failed = $terlambat = $complete = $waiting = 0;
        $gaps_status = 0;
        
        $title_status=["On Progress","Failed","SUSPENDED","Completed", "Waiting"];

        $estimate_delivery = $total_project = $gap = $gap_status = $Planning = $Procurement = $Implementation = $Closing = 0;
        $progress = $progress_status = 0;
        $gap_cnt = $gap_status_cnt = $Planning_cnt = $Procurement_cnt = $Implementation_cnt = $Closing_cnt = 0;
        $progress_cnt = $gap_lbh_20_cnt = $gap_lbh_10_cnt = $gap_kurang_10_cnt = 0;
        $gap_note = 'ON TRACK';
        $gap_color = 'bg-success';

        $data_progress = array(
            'progress_status' => array(
                'Planning' => 0,
                'Procurement' => 0,
                'Implementation' => 0,
                'Closing' => 0,
            ),
            'progress' => 0,
            'estimate_delivery' => 0,
            'gap_status' => 0,
            'gap_color' => $gap_color,
            'gap_note' => $gap_note
        );

        $gaps = array(
            'estimate_delivery' => 0,
            'gap_status' => 0,
            'gap_color' => $gap_color,
            'gap_note' => $gap_note
        );

        $project_names = $nama = $realisasi = $target = array();
        if($projects->isNotEmpty()){
            $project_ids = $projects->pluck('id');
            $total_project = $projects->count();

            //gap status
            $sub = GanttTaskResumeDaily::whereIn('project_id', $project_ids)
                ->whereNotNull('progress_daily_akumulasi_realisasi')
                ->selectRaw('project_id, max(days) as max_days')
                ->groupBy('project_id');

            $gap_status = GanttTaskResumeDaily::whereIn('gantt_task_resume_dailies.project_id', $project_ids)
                ->joinSub($sub, 'sub', function($j){
                    $j->on('sub.project_id','gantt_task_resume_dailies.project_id')
                        ->on('sub.max_days','gantt_task_resume_dailies.days');
                })
                ->join('projects','projects.id','gantt_task_resume_dailies.project_id')
                ->selectRaw("projects.project_name,gantt_task_resume_dailies.project_id, (progress_daily_akumulasi_realisasi - progress_daily_akumulasi_planning) as dif_progress,progress_daily_akumulasi_realisasi as realisasi, progress_daily_akumulasi_planning as planning")
                ->get();
            if($gap_status->isNotEmpty()){                
                $progress_status = $gap_status->pluck('realisasi')->sum();
                foreach($gap_status as $gproject){
                    $nama[] = $gproject->project_name;
                    $realisasi[] = round($gproject->realisasi);
                    $target[]= round($gproject->planning);
                    $gaps_status += round($gproject->realisasi) - round($gproject->planning);
                }            
                
                $gap = round($gaps_status/$gap_status->count());
                if($gap < -20){
                    $gap_color = 'bg-danger';
                    $gap_note = 'RISKS / ROADBLOCKS';
                }elseif($gap <= -10){
                    $gap_color = 'bg-warning';
                    $gap_note = 'POTENTIAL RISKS';
                }

            }
            
            $gap_q = GanttTask::join('gantt_task_actuals', 'gantt_task_actuals.project_id', 'gantt_tasks.project_id')
                ->where('gantt_task_actuals.sortable', 'gantt_tasks.sortable')
                ->whereIn('gantt_tasks.project_id', $project_ids)
                ->where('gantt_tasks.sortable', 0)
                ->selectRaw('gantt_tasks.end planning_end,gantt_tasks.duration planning_duration,
                    gantt_task_actuals.progress actual_progress,gantt_task_actuals.end actual_end,gantt_task_actuals.duration actual_duration')
                ->get();
            if($gap_q->isNotEmpty()){
                
                foreach($gap_q as $val_gap){
                    $est_gap = Carbon::parse($val_gap->planning_end)->diffInDays($val_gap->actual_end,false);
                    if($est_gap > 0){
                        $estimate_delivery += $est_gap;
                    }
                }
            }
            
           

            foreach($projects as $val){
                
                if($val->status != 'Started'){
                    $project_names[] = '<span class="badge bg-danger">'.$val->project_name.'</span> <span class="badge" style="color: #dededf;">(Not Started)</span>';
                }
                if($total_project == 1){
                    if($val->status == 'Started'){
                        $aktual = $val->GanttaskActual->where('sortable','>',0);
                        $progress = $aktual->where('status','STATUS_ACTIVE')->count();
                        $failed = $aktual->where('status','STATUS_FAILED')->count();
                        $terlambat = $aktual->where('status','STATUS_SUSPENDED')->count();
                        $complete = $aktual->where('status','STATUS_DONE')->count();
                        $waiting = $aktual->whereIn('status',['STATUS_UNDEFINED','STATUS_WAITING'])->count();

                        $aktual = $val->GanttaskActual->where('sortable','>',1);
                        $Planning = ( $aktual->where('code','Planning')->count() == 0?0:round($aktual->where('code','Planning')->sum('progress')/$aktual->where('code','Planning')->count()) );
                        $Procurement = ( $aktual->where('code','Procurement')->count() == 0?0:round($aktual->where('code','Procurement')->sum('progress')/$aktual->where('code','Procurement')->count()) );
                        $Implementation = ( $aktual->where('code','Implementation')->count() == 0?0:round($aktual->where('code','Implementation')->sum('progress')/$aktual->where('code','Implementation')->count()) );
                        $Closing = ( $aktual->where('code','Closing')->count() == 0?0:round($aktual->where('code','Closing')->sum('progress')/$aktual->where('code','Closing')->count()) );
                        
                        $project_names[] = '<span class="badge bg-danger">'.$val->project_name.'</span>';
                        if($request->has('project_id')){
                            $project_names_s[] = '<h4><span class="badge bg-danger">'.$val->project_name.'</span></h4>';
                        }
                    }
                }else{
                    if($val->status == 'Started'){
                        if($val->GanttaskActual->first()->status == 'STATUS_ACTIVE'){
                            ++$progress;
                            $status_p = '<span class="badge" style="color: #3BBF67;">on progress</span>';
                        }elseif($val->GanttaskActual->first()->status == 'STATUS_FAILED'){
                            ++$failed;
                            $status_p = '<span class="badge" style="color: #763A96;">failed</span>';
                        }elseif($val->GanttaskActual->first()->status == 'STATUS_SUSPENDED'){
                            ++$terlambat;
                            $status_p = '<span class="badge" style="color: #F9C154;">suspended</span>';
                        }elseif($val->GanttaskActual->first()->status == 'STATUS_DONE'){
                            ++$complete;
                            $status_p = '<span class="badge" style="color: #6EBEF4;">done</span>';
                        }elseif(in_array($val->GanttaskActual->first()->status, ['STATUS_UNDEFINED','STATUS_WAITING']) ){
                            ++$waiting;
                            $status_p = '<span class="badge" style="color: #dededf;">waiting</span>';
                        }
                        
                        $aktual = $val->GanttaskActual->where('sortable','>',1);
                        $Planning += ($aktual->where('code','Planning')->count() == 0?0:round($aktual->where('code','Planning')->sum('progress')/$aktual->where('code','Planning')->count()));
                        $Procurement += ($aktual->where('code','Procurement')->count() == 0?0:round($aktual->where('code','Procurement')->sum('progress')/$aktual->where('code','Procurement')->count()));
                        $Implementation += ($aktual->where('code','Implementation')->count() == 0?0:round($aktual->where('code','Implementation')->sum('progress')/$aktual->where('code','Implementation')->count()));
                        $Closing += ($aktual->where('code','Closing')->count() == 0?0:round($aktual->where('code','Closing')->sum('progress')/$aktual->where('code','Closing')->count()));

                        $project_names[] = '<span class="badge bg-danger">'.$val->project_name.'</span> ('.$status_p.')';
                        if($request->has('project_id')){
                            $project_names_s[] = '<h4><span class="badge bg-danger">'.$val->project_name.'</span></h4>';
                        }
                    }
                }
            }

            $data_progress = array(
                'progress_status' => array(
                    'Planning' => $Planning?round($Planning/$total_project):0,
                    'Procurement' => $Procurement?round($Procurement/$total_project):0,
                    'Implementation' => $Implementation?round($Implementation/$total_project):0,
                    'Closing' => $Closing?round($Closing/$total_project):0,
                ),
                'progress' => round($progress_status/$total_project),
                'estimate_delivery' => round($estimate_delivery/$total_project),
                'gap_status' => (substr($gap,0,1)=="-"? $gap: "+".$gap),
                'gap_color' => $gap_color,
                'gap_note' => $gap_note
            );
        }
        
        $dat=[$progress,$failed,$terlambat,$complete, $waiting];
        
        $costs = Project::leftJoin('gantt_tasks','projects.id','gantt_tasks.project_id')
            ->leftJoin('gantt_task_actuals','projects.id','gantt_task_actuals.project_id')
            ->where('gantt_task_actuals.level',0)
            ->where('gantt_tasks.level',0)
            ->when(!empty($request->project_id), function($w)use($request){
                $w->whereIn('projects.id',$request->project_id);
            })                         
            ->when($request->filled('start_date') and $request->filled('end_date'), function($w) use($start_date, $end_date){
                $w->whereBetween('start_date',[$start_date.' 00:00:00', $end_date.' 23:59:59'])->orwhereBetween('end_date',[$start_date.' 00:00:00', $end_date.' 23:59:59']);
                
            })
            ->when(count($request->all()) == 0, function($w) use($start_year, $end_year){
                $w->whereBetween('start_date',[$start_year.' 00:00:00', $end_year.' 23:59:59'])->orwhereBetween('end_date',[$start_year.' 00:00:00', $end_year.' 23:59:59']);
                
            })                       
            ->selectRaw("projects.project_name, SUM(gantt_task_actuals.task_budget) as actual, SUM(gantt_tasks.task_budget) as plan,
                projects.project_budget as budget")
            ->groupBy('projects.id')->get();

        //status proyek


        $costnama=[];
        $costactual=[];
        $costplan=[];
        $costbudget=[];
        $i = 1;
        foreach($costs as $cost)
        {
            $costnama[]=$cost->project_name;
            $costactual[]=(int)$cost->actual;
            $costplan[]=(int)$cost->plan;
            $costbudget[]=$cost->budget;

        }

        $sizechart = array(
            'minwidth' => "800px",
            'maxwidth' => "1000px",
            'width' => "900px",
            'chartminWidth' => "900"
        );
        if(count($costnama)>9){
            $sizechart = array(
                'minwidth' => "820px",
                'maxwidth' => "1900px",
                'width' => "1010px",
                'chartminWidth' => "2000"
            );
            
        }
        
        
        return view('admin.dashboard.seeallCost',compact('data','costnama','costactual','costplan','costbudget','sizechart'));
    }
}
