<?php

namespace App\Http\Controllers;

use App\Models\Employee;
use App\Models\User;
use Illuminate\Http\Request;
use Spatie\Permission\Models\Role;
use DB;
use Hash;
use Illuminate\Support\Arr;
use Illuminate\Support\Facades\Auth;
use Illuminate\Auth\Events\Registered;

class UserController extends Controller
{
    public function __construct()
    {
        return $this->middleware('auth:web'); 
    }
    
    public function index()
    {
        $data = User::orderBy('id','DESC')->get();
        $role = Role::get();
        return view('role_pengguna.pengguna.index',compact('data','role'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $this->validate($request, [
            'email' => 'required|email|unique:users,email',
            'position' => 'required',
            'name' => 'required'
        ],[
            'email.unique' => 'User/Email Exist!'
        ]);

        // $input = $request->all();
        $input['name'] = strtoupper($request->name);
        $input['password'] = bcrypt($request->password);
        $input['username'] = $request->email;
        $input['email'] = $request->email;
        // dd($input);
        $user = User::create($input);
        $user->assignRole($request->input('position'));

        event(new Registered($user));

        $rolename = Role::where('id',$request->position)->select('name')->first();
        $employee = new Employee();
        $employee->user_id = $user->id;
        $employee->name = strtoupper($request->name);
        $employee->email = $user->email;
        $employee->position = $request->position;
        $employee->position_title = $rolename->name;
        $employee->save();
        return redirect()->route('user.index')->withSuccess('User Added');
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $user = User::findOrFail(decrypt($id));
        $roles = Role::get();
        // $roles = Role::pluck('name','name')->all();
        $userRole = $user->roles->pluck('name','name')->all();
    
        return view('role_pengguna.pengguna.edit',compact('user','roles','userRole'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        $this->validate($request, [
            'email' => 'required|email|unique:users,email,'.$id,
            'roles' => 'required'
        ]);
    
        // $input = $request->all();
        $user = User::findOrFail($id);
        $user->is_active = $request->is_active;
        $user->save();
        // $user->update($input);
        DB::table('model_has_roles')->where('model_id',$id)->delete();
    
        $user->assignRole($request->input('roles'));
    
        // dd($user);
        return redirect()->route('user.index')->with('success','Succesfully Updated');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        //
    }

    public function logout(Request $request) {
        Auth::logout();
        return redirect('/');
    }
}
