<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;

class Project extends Model
{
    use HasFactory, SoftDeletes;

    protected $table = 'projects';
    protected $dates = ['deleted_at'];
    protected $guarded = [];
 
    

    protected $fillable = [
        'step',
        'jabatanid',
        'sso_divisi',
        'nama',
        'divisi_id',
        'wbs_id',
        'portofolio',
        'grouptag',
        'series_id',
        'project_code',
        'full_number',
        'project_name',
        'type_projeect',
        'project_budget',
        'investasi',
        'cost',
        'project_description',
        'nama_po',
        'project_owner',
        'jabatan_po',
        'nama_pm',
        'project_manager',
        'jabatan_pm',
        'team_lead',
        'team_member',
        'start_date',
        'end_date',
        'duration',
        'status',
        'send_approved_date',
        'date_rejected_manager',
        'date_rejected_vp',
        'date_approved_manager',
        'date_approved_vp',
        'reason_rejected',
        'target_waktu',
        'project_doc',
        'risk_doc',
        'created_by',
        'updated_by',
        'deleted_by',
        'gantt_updated_at',
        'gantt_updated_by'
    ];

    public function employee()
    {
        return $this->hasOne(Employee::class,'user_id','created_by');
    }

    

    public function createdby()
    {
        return $this->hasOne(User::class,'id','created_by');
    }

    public function updatedby()
    {
        return $this->hasOne(User::class,'id','updated_by');
    }

    public function ganttupdatedby()
    {
        return $this->belongsTo(User::class,'gantt_updated_by','id');
    }

    public function deletedby()
    {
        return $this->hasOne(User::class,'id','deleted_by');
    }

    public function series()
    {
        return $this->belongsTo(Series::class,'series_id');
    }

    protected static function boot(){
        parent::boot();
    
        static::creating(function($model){
            $model->project_code = Project::where('series_id',$model->series_id)->max('project_code') + 1;
            $model->full_number = $model->series->prefix . '-' . str_pad($model->project_code,5,'0',STR_PAD_LEFT);
        });
    }

    public function Wbs()
    {
        return $this->hasMany(Wbs::class);
    }

    public function Ganttask()
    {
        return $this->hasMany(GanttTask::class)->orderBy('sortable');
    }

    public function GanttaskActual()
    {
        return $this->hasMany(ActualGanttTask::class)->orderBy('sortable');
    }

    public function getLastRejected(){
        return $this->hasOne(Approval::class)->where('status','like','Project ditolak%')->orderBy('id','desc');
    }
}
