<?php

namespace App\Notifications;

use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Notifications\Messages\MailMessage;
use Illuminate\Notifications\Notification;

class ApprovalNotification extends Notification implements ShouldQueue
{
    use Queueable;
    // private $project_name,$status, $type, $project, $user, $created,$userpmo;
    private $project_name,$status, $type, $project, $user;

    /**
     * Create a new notification instance.
     *
     * @return void
     */
    // public function __construct($project,$project_name, $status, $type,$user, $created,$userpmo)
    public function __construct($project,$project_name, $status, $type,$user)
    {
        $this->project = $project;
        $this->project_name = $project_name;
        $this->status = $status;
        $this->type = $type;
        $this->user = $user;
        // $this->created = $created;
        // $this->userpmo = $userpmo;
    }

    /**
     * Get the notification's delivery channels.
     *
     * @param  mixed  $notifiable
     * @return array
     */
    
    /**
     * Get the mail representation of the notification.
     *
     * @param  mixed  $notifiable
     * @return \Illuminate\Notifications\Messages\MailMessage
     */
    public function via($notifiable)
    {
        return ['database','mail'];
    }

    public function toMail($notifiable)
    {
        if ($this->type == 'Staf' and ($this->status == 'draft' or $this->status == 'Project has been made'))
        {
            return (new MailMessage)

                    ->subject("Project of ".$this->project_name." has been made")
                    ->view('vendor.mail.mail',[
                        'content'   => $this->project,
                        'nama_project' => $this->project_name,
                        'nama'  => $this->user->name,
                        'url_project' => env('APP_URL'),
                    ]);
        }
        elseif ($this->type == 'Staf' and $this->status == 'Waiting for project manager approval')
        {
            return (new MailMessage)

                    ->subject("Request for Approval ".$this->project_name)
                    ->view('vendor.mail.waitapproval',[
                        'content'   => $this->project,
                        'nama_project' => $this->project_name,
                        'nama_manager'  => $this->project->nama_pm,
                        'email_manager'  => $this->project->project_manager,
                        'project_maker'  => $this->project->name,
                        'project_made'  => $this->project->updated_at,
                        'url_project' => env('APP_URL'),
                    ]);
        }
        elseif ($this->type == 'Manager' and $this->status == 'Waiting for project owner approval')
        {
            return (new MailMessage)

                    ->subject("Request for Approval ".$this->project_name)
                    ->view('vendor.mail.waitapprovaltovp',[
                        'content'   => $this->project,
                        'nama_project' => $this->project_name,
                        'nama_project_owner'  => $this->project->nama_po,
                        'email_project_owner'  => $this->project->project_owner,
                        'project_manager'  => $this->project->nama_pm,
                        'project_made'  => $this->project->updated_at,
                        'url_project' => env('APP_URL'),
                    ]);
        }
        elseif ($this->type == 'VP' and $this->status == 'ProjectOwnerApproved')
        {
            return (new MailMessage)

                    ->subject("Project of ".$this->project_name. " has been Approved by Project Owner")
                    ->view('vendor.mail.vpapproved',[
                        'content'   => $this->project,
                        'nama_project' => $this->project_name,
                        'nama_project_owner'  => $this->project->nama_po,
                        'email_project_owner'  => $this->project->project_owner,
                        'project_manager'  => $this->project->nama_pm,
                        'project_made'  => $this->project->updated_at,
                        'url_project' => env('APP_URL'),
                    ]);
        }
        elseif (strtolower($this->type) == 'staf' and $this->status == 'Started')
        {
            return (new MailMessage)

                    ->subject("Project of ".$this->project_name." has been started")
                    ->view('vendor.mail.projectstarted',[
                        'content'   => $this->project,
                        // 'nama'  => $this->created,
                        'url_project' => env('APP_URL'),
                    ]);
        }
        elseif($this->type == 'Manager' and $this->status == 'Project rejected by Manager')
        {
            return (new MailMessage)

                    ->subject("Project of ".$this->project_name." has been rejected")
                    ->view('vendor.mail.rejectedmanager',[
                        'content'   => $this->project,
                        'nama_project' => $this->project_name,
                        'date_rejected_manager' => $this->project->date_rejected_manager,
                        'reason_rejected' => $this->project->reason_rejected,
                        'project_manager'  => $this->project->nama_pm,
                        'nama'  => $this->user->name,
                        'url_project' => env('APP_URL'),
                    ]);
        }
        elseif($this->type == 'VP' and $this->status == 'Project rejected by VP')
        {
            return (new MailMessage)

                    ->subject("Project of ".$this->project_name." has been rejected")
                    ->view('vendor.mail.rejectedvp',[
                        'content'   => $this->project,
                        'nama_project' => $this->project_name,
                        'date_rejected_manager' => $this->project->date_rejected_vp,
                        'reason_rejected' => $this->project->reason_rejected,
                        'project_manager'  => $this->project->nama_po,
                        'nama'  => $this->user->name,
                        'url_project' => env('APP_URL'),
                    ]);
        }
        
    }

    

    /**
     * Get the array representation of the notification.
     *
     * @param  mixed  $notifiable
     * @return array
     */
    public function toArray($notifiable)
    
    {
        if (strtolower($this->type) == 'staf' and ($this->status == 'draft' or $this->status == 'Project has been made'))
        {
            return [
                'data' => 'Project name : '.$this->project_name.' has been made',

                'name'      => 'Created by : '.$this->project->createdby->name,   
                'url'   => '/project',
                'created_date'  => 'Created at : '.$this->project->created_at,
         
            ];
        }
        elseif (strtolower($this->type) == 'staf' and $this->status == 'Waiting for project manager approval')
        {
            return [
                'data' => 'Project name : '.$this->project_name,

                'name'      => 'Request for Approval by : '.$this->project->nama,   
                'url'   => '/project',
                'created_date'  => 'Request sent at : '.$this->project->send_approved_date,
           
            ];
        }
        elseif($this->type == 'Manager' and $this->status == 'Waiting for project owner approval')
        {
            return [
                'data' => $this->project_name,

                'name'      => 'Request for Approval by : '.$this->project->nama_pm,   
                'url'   => '/project',
                'created_date'  => 'Request sent at : '.$this->project->date_approved_manager,

            ];
        }
        elseif((strtolower($this->type) == 'staf' or strtoupper($this->type) == 'VP') and $this->status == 'ProjectOwnerApproved')
        {
            return [
                'data' => $this->project_name,

                'name'      => 'Submitted by : '.$this->project->createdby->name,   
                'url'   => '/project',

            ];
        }
        elseif((strtolower($this->type) == 'staf' or strtoupper($this->type) == 'VP') and $this->status == 'Started')
        {
            return [
                'data' => $this->project_name,

                'name'      => 'Submitted by : '.$this->project->createdby->name,   
                'url'   => '/project',

            ];
        }
        elseif($this->type == 'Manager' and $this->status == 'Project rejected by Manager')
        {
            return [
                'data' => $this->project_name.' has been rejected by Project Manager!',

                'name'      => 'Rejected by : '.$this->project->nama_pm, 
                'created_date' => 'Rejected at : '.$this->project->date_rejected_manager,  
                'url'   => '/project',

            ];
        }
        elseif($this->type == 'VP' and $this->status == 'Project rejected by VP')
        {
            return [
                'data' => $this->project_name.' has been rejected by Project Owner!',

                'name'      => 'Rejected by : '.$this->project->nama_po, 
                'created_date' => 'Rejected at : '.$this->project->date_rejected_vp,  
                'url'   => '/project',

            ];
        }
    }
}
