<?php

namespace App\Notifications;

use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Notifications\Messages\MailMessage;
use Illuminate\Notifications\Notification;

class RejectNotification extends Notification implements ShouldQueue
{
    use Queueable;
    private $project_name,$status, $type, $project, $user, $created,$vp;

    /**
     * Create a new notification instance.
     *
     * @return void
     */
    public function __construct($project,$project_name, $status, $type,$user, $created,$vp)
    {
        $this->project = $project;
        $this->project_name = $project_name;
        $this->status = $status;
        $this->type = $type;
        $this->user = $user;
        $this->created = $created;
        $this->vp = $vp;
        
    }

    /**
     * Get the notification's delivery channels.
     *
     * @param  mixed  $notifiable
     * @return array
     */
    public function via($notifiable)
    {
        return ['database','mail'];
    }

    /**
     * Get the mail representation of the notification.
     *
     * @param  mixed  $notifiable
     * @return \Illuminate\Notifications\Messages\MailMessage
     */
    public function toMail($notifiable)
    {
        if ($this->type == 'Manager' || $this->status == 'Project ditolak Manager')
        {
            return (new MailMessage)
                    // ->line('The introduction to the notification.')
                    // ->action('Notification Action', url('/'))
                    // ->line('Thank you for using our application!');
                    ->subject("You Reject the Application ".$this->project_name)
                    ->view('vendor.mailreject.rejected',[
                        'content'   => $this->project,
                        'nama'  => $this->user,
                    ]);
        }
        elseif ($this->type == 'ProjectditolakManager' || $this->status == 'ProjectditolakManager')
        {
            return (new MailMessage)
                    // ->line('The introduction to the notification.')
                    // ->action('Notification Action', url('/'))
                    // ->line('Thank you for using our application!');
                    ->subject("Request ".$this->project_name." Di Tolak Manager")
                    ->view('vendor.mailreject.rejectedmgrtouser',[
                        'content'   => $this->project,
                        'nama'  => $this->created,
                    ]);
        }
        elseif ($this->type == 'VP' || $this->status == 'Project ditolak VP')
        {
            return (new MailMessage)
                    // ->line('The introduction to the notification.')
                    // ->action('Notification Action', url('/'))
                    // ->line('Thank you for using our application!');
                    ->subject("Request ".$this->project_name." Di Tolak VP")
                    ->view('vendor.mailreject.rejectedvptouser',[
                        'content'   => $this->project,
                        'nama'  => $this->created,
                    ]);
        }
        elseif ($this->type == 'ProjectditolakVP' || $this->status == 'ProjectditolakVP')
        {
            return (new MailMessage)
                    // ->line('The introduction to the notification.')
                    // ->action('Notification Action', url('/'))
                    // ->line('Thank you for using our application!');
                    ->subject("Request ".$this->project_name." Di Tolak VP")
                    ->view('vendor.mailreject.rejectedvptomgr',[
                        'content'   => $this->project,
                        'nama'  => $this->user,
                    ]);
        }
        elseif ($this->type == 'Projectditolak' || $this->status == 'Projectditolak')
        {
            return (new MailMessage)
                    // ->line('The introduction to the notification.')
                    // ->action('Notification Action', url('/'))
                    // ->line('Thank you for using our application!');
                    ->subject("You Reject the Application ".$this->project_name)
                    ->view('vendor.mailreject.rejected',[
                        'content'   => $this->project,
                        'nama'  => $this->vp,
                    ]);
        }
    }

    /**
     * Get the array representation of the notification.
     *
     * @param  mixed  $notifiable
     * @return array
     */
    public function toArray($notifiable)
    {
        if($this->type == 'Manager' && $this->status == 'Project ditolak Manager')
        {
            return [
                'data' => $this->project_name,
                // 'title'     => 'Persetujuan Project'.$this->project->project_name,
                // 'name'      => 'Diajukan oleh : '.$this->project->createdby->name,   
                'name'      => "Request ".$this->project_name." Di Tolak Manager",//'Diajukan oleh : '.$this->project->createdby->name,   
                'url'   => '/project',
                // 'title'  => $this->datasend['title'], 
            ];
        }
        elseif($this->type == 'ProjectditolakManager' && $this->status == 'ProjectditolakManager')
        {
            return [
                'data' => $this->project_name,
                // 'title'     => 'Persetujuan Project'.$this->project->project_name,
                // 'name'      => 'Diajukan oleh : '.$this->project->createdby->name,   
                'name'      => "Request ".$this->project_name." Di Tolak Manager",
                'url'   => '/project',
                // 'title'  => $this->datasend['title'], 
            ];
        }
        elseif($this->type == 'VP' && $this->status == 'Project ditolak VP')
        {
            return [
                'data' => $this->project_name,
                // 'title'     => 'Persetujuan Project'.$this->project->project_name,
                // 'name'      => 'Diajukan oleh : '.$this->project->createdby->name,   
                'name'      => "Permohonan ".$this->project_name." Di Tolak VP",//
                'url'   => '/project',
                // 'title'  => $this->datasend['title'], 
            ];
        }
        elseif($this->type == 'ProjectditolakVP' && $this->status == 'ProjectditolakVP')
        {
            return [
                'data' => $this->project_name,
                // 'title'     => 'Persetujuan Project'.$this->project->project_name,
                // 'name'      => 'Diajukan oleh : '.$this->project->createdby->name,   
                'name'      => "Request ".$this->project_name." Di Tolak VP",//
                'url'   => '/project',
                // 'title'  => $this->datasend['title'], 
            ];
        }
        elseif($this->type == 'Projectditolak' && $this->status == 'Projectditolak')
        {
            return [
                'data' => $this->project_name,
                // 'title'     => 'Persetujuan Project'.$this->project->project_name,
                'name'      => 'Diajukan oleh : '.$this->project->createdby->name,   
                'url'   => '/project',
                // 'title'  => $this->datasend['title'], 
            ];
        }
    }
}
